package com.link.core.converter.util;

import java.lang.reflect.Constructor;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

public class ExceptionUtil {
   private static final ResourceBundle resources = ResourceBundleUtil.getBundle(ExceptionUtil.class, "R");
   private static final Class[][] EXCEPTION_CONSTRUCTOR_PARAMETERS_PATTERNS = new Class[][]{{String.class, Throwable.class, Object[].class}, {String.class, Throwable.class}, {Throwable.class, String.class, Object[].class}, {Throwable.class, String.class}, {String.class, Object[].class}, {String.class}, {Throwable.class, Object[].class}, {Throwable.class}, new Class[0]};

   public static IllegalArgumentException nullArgument(String argument) {
      return new IllegalArgumentException(MessageFormat.format(resources.getString("nullArgument"), argument));
   }

   public static IllegalStateException nullAttribute(String attribute) {
      return new IllegalStateException(MessageFormat.format(resources.getString("nullAttribute"), attribute));
   }

   public static UnsupportedOperationException neverCallMe(String methodName) {
      return new UnsupportedOperationException(MessageFormat.format(resources.getString("neverCallMe"), methodName));
   }

   public static IllegalArgumentException stringTooLong(String argument, long maxLength) {
      return new IllegalArgumentException(MessageFormat.format(resources.getString("stringTooLong"), argument, maxLength));
   }

   public static ExceptionBean toBean(Throwable t) {
      return toBean(t, SystemEx.isEnableAssertions());
   }

   public static ExceptionBean toBean(Throwable t, boolean includeStackTrace) {
      if (t == null) {
         return null;
      } else {
         ExceptionBean bean = new ExceptionBean();
         if (t instanceof StandInException) {
            bean.setExceptionClass(((StandInException)t).getOriginalExceptionClass());
         } else {
            bean.setExceptionClass(t.getClass().getName());
         }

         bean.setMessage(t.getMessage());
         if (includeStackTrace) {
            List<StackTraceBean> stackTrace = new ArrayList();
            StackTraceElement[] var4 = t.getStackTrace();
            int var5 = var4.length;

            for(int var6 = 0; var6 < var5; ++var6) {
               StackTraceElement element = var4[var6];
               StackTraceBean traceBean = new StackTraceBean(element);
               stackTrace.add(traceBean);
            }

            bean.setStackTrace(stackTrace);
         }

         bean.setCause(toBean(t.getCause(), includeStackTrace));
         return bean;
      }
   }

   public static Throwable toThrowable(ExceptionBean bean) {
      return toThrowable(bean, (ClassLoader)null, (Class)null);
   }

   public static <E extends StandInException> Throwable toThrowable(ExceptionBean bean, Class<E> standInExceptionClass) {
      return toThrowable(bean, (ClassLoader)null, standInExceptionClass);
   }

   public static <E extends StandInException> Throwable toThrowable(ExceptionBean bean, ClassLoader classLoader, Class<E> standInExceptionClass) {
      if (bean == null) {
         return null;
      } else {
         ClassLoader cl = classLoader == null ? ExceptionUtil.class.getClassLoader() : classLoader;
         Throwable cause = null;
         if (bean.getCause() != null) {
            cause = toThrowable(bean.getCause(), cl, standInExceptionClass);
         }

         Throwable t = newException(cl, bean.getExceptionClass(), bean.getMessage(), cause);
         if (t == null) {
            t = newException(standInExceptionClass, bean.getMessage(), cause);

            assert t != null;

            ((StandInException)t).setOriginalExceptionClass(bean.getExceptionClass());
         }

         injectStackTrace(t, bean.getStackTrace());
         return t;
      }
   }

   private static Throwable newException(ClassLoader classLoader, String className, String message, Throwable cause) {
      assert classLoader != null;

      assert className != null;

      try {
         Class cls = classLoader.loadClass(className);
         return !ClassGetAllSuperclasses(cls).contains(Exception.class) ? null : newException(cls, message, cause);
      } catch (Exception var5) {
         return null;
      }
   }

   private static Throwable newException(Class exceptionClass, String message, Throwable cause) {
      if (exceptionClass != null && !StandInException.class.equals(exceptionClass)) {
         Class[][] var16 = EXCEPTION_CONSTRUCTOR_PARAMETERS_PATTERNS;
         int var4 = var16.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            Class[] pattern = var16[var5];
            Constructor<Exception> c = null;

            try {
               c = exceptionClass.getConstructor(pattern);
            } catch (NoSuchMethodException var15) {
               continue;
            }

            List<Object> paramList = new ArrayList();
            boolean causeUsed = false;
            Class[] var10 = pattern;
            int var11 = pattern.length;

            for(int var12 = 0; var12 < var11; ++var12) {
               Class type = var10[var12];
               if (type == String.class) {
                  paramList.add(message);
               } else if (type == Throwable.class) {
                  paramList.add(cause);
                  causeUsed = true;
               } else if (type == Object[].class) {
                  paramList.add(new Object[0]);
               } else {
                  assert false;
               }
            }

            try {
               Throwable t = (Throwable)c.newInstance(paramList.toArray());
               if (!causeUsed && cause != null) {
                  t.initCause(cause);
               }

               return t;
            } catch (Exception var14) {
            }
         }

         return null;
      } else {
         Throwable t = new StandInException(message);
         if (cause != null) {
            t.initCause(cause);
         }

         return t;
      }
   }

   public static List<Class<?>> ClassGetAllSuperclasses(Class<?> cls) {
      if (cls == null) {
         return null;
      } else {
         List<Class<?>> classes = new ArrayList();

         for(Class superclass = cls.getSuperclass(); superclass != null; superclass = superclass.getSuperclass()) {
            classes.add(superclass);
         }

         return classes;
      }
   }

   private static void injectStackTrace(Throwable t, List<StackTraceBean> stackTrace) {
      assert t != null;

      assert stackTrace != null;

      if (stackTrace.isEmpty()) {
         t.setStackTrace(new StackTraceElement[0]);
      } else {
         StackTraceElement[] elements = new StackTraceElement[stackTrace.size()];

         for(int index = 0; index < stackTrace.size(); ++index) {
            StackTraceBean bean = (StackTraceBean)stackTrace.get(index);

            assert bean != null;

            elements[index] = bean.toStackTraceElement();
         }

         t.setStackTrace(elements);
      }
   }
}
